// Strings.h
#ifndef Strings_h
#define Strings_h

#ifndef ssize_t
typedef SSIZE_T ssize_t;             // ssize_t is defined in GCC, but isn't defined in VC9-15, but rather SSIZE_T.  For symetry we'll define it.
#endif
#include "stdio.h"

#define MINIMUM_ALLOCATION     16    // allocate at least this for every String
#define EXPANSION_FACTOR        2    // repeated concatenations will keep doubling buffer
#define NUMERIC_CONVERSIONS          // allows direct assignment of numeric values to Strings, e.g., String s(3.14159) or s=3.14159;
#define FORMAT_MONEY                 // put commas every three places, rounding, left/right justification, specify field sizes (padding), etc.
#define CONSOLE_OUTPUT               // support console output, i.e., enable String::Print()
#define                       x64

class String
{
 public:                                        // Constructors (8)
 String();                                      // Uninitialized Constructor
 String(const int iSize, bool blnNullOut);      // Constructor creates String of size iSize and optionally nulls out
 String(const char ch);                         // Constructor creates a String initialized with a char, e.g., String c('A');
 String(const char* pStr);                      // Constructor: Initializes with char*, e.g. s1 = "PowerBASIC! Compile Without Compromise!"
 String(const String& strAnother);              // Constructor creates String initialized with another already existing String, e.g., String s2(s1);
 #ifdef NUMERIC_CONVERSIONS
    String(int iNumber);                        // Constructor creates String initialized with an int, e.g., String s1(2468); kind of Str$(2468) in PowerBASIC
    String(unsigned int uNumber);               // Constructor creates String initialized with an unsigned int, e.g., String s1(2468); kind of Str$(2468) in PowerBASIC
    #ifdef x64
       String(size_t  uiNumber);                // Constructor creates String from 64 bit unsigned number, e.g., String strBigNum(12345678987654);
       String(ssize_t iNumber);                 // Constructor creates String from 64 bit signed number, e.g., String strBigNum(-12345678987654);
    #endif
    String(double dblNumber);                   // Constructor creates String from floating point number, e.g., String s(3.14159);
 #endif
 String& operator=(const char c);               // Assign a char to a String, e.g., String s='A';
 String& operator=(const char* pChar);          // Assign a character string to a String Object, e.g.,  String s1 = "Compile Without Compromise";
 String& operator=(const String& strAnother);   // Assign an already existing String to this one, e.g., String s2 = s1;
 #ifdef NUMERIC_CONVERSIONS
    String& operator=(int iNumber);             // Assign an          int converted to a String to this, e.g., String s1 = -123456789;
    String& operator=(unsigned int uNumber);    // Assign an unsigned int converted to a String to this, e.g., String s1 =  123456789;
    #ifdef x64
       String& operator=(size_t  uNumber);      // Assign a 64 bit unsigned quantity converted to a String to this, e.g., String s2 =  12345678987654;
       String& operator=(ssize_t iNumber);      // Assign a 64 bit   signed quantity converted to a String to this, e.g., String s2 = -12345678987654;
    #endif
    String& operator=(double dblNumber);        // Assign a double converted to a String to this, e.g., String strDouble = 3.14159;
 #endif
 String operator+(const TCHAR ch);              // Concatenates or adds a character to an already existing String, e.g., s1 = s1 + 'A';
 String operator+(const TCHAR* pChar);          // Concatenates or adds a character array (char*) to an already existing String, e.g., s1 = s1 + lpText;
 String operator+(String& s);                   // Concatenates or adds another String to this one, e.g., s1 = s1 + s2;
 bool operator==(String& s);                    // Compares two Strings for case sensitive equality
 bool operator==(const char* pChar);            // Compares a String against a char* for case sensitive equality
 bool operator!=(char* pChar);                  // Compares a String against a char* for case sensitive inequality
 void LTrim();                                  // Removes leading white space by modifying existing String
 void RTrim();                                  // Removes trailing white space by modifying existing String
 void Trim();                                   // Removes leading or trailing white space from existing String
 String Left(size_t iCntChars);                 // Returns a String consisting of left-most iCntChars of this
 String Right(size_t iCntChars);                // Returns a String consisting of right-most iCntChars of this
 String Mid(size_t iStart, size_t iCount);      // Returns a String consisting of iCount characters of this starting at one based iStart
 int InStr(const char* pStr, bool blnCaseSensitive, bool blnStartLeft);   // Returns one based position of pStr in this by case sensitivity and left/right start
 int InStr(const String& str, bool blnCaseSensitive, bool blnStartLeft);  // Returns one based position of String in this by case sensitivity and left/right start
 int ParseCount(const char ch);                                           // Returns count of delimited fields as specified by char delimiter, i.e., comma, space, tab, etc.
 void Parse(String* pStrs, char cDelimiterh, size_t iParseCount);         // Parses this based on delimiter.  Must pass in 1st parameter String* to sufficient number of Strings
 String Replace(char* pToRevove, char* pReplacement);                     // Replaces pToRemove with pReplacement in new String.  Replacement can cause String to grow
 String Remove(const char* pCharsToRemove);                               // Returns A String With All The chars In A char* Removed (Individual char removal)
 String Remove(const char* pStrToRemove, bool bCaseSensitive);            //Returns a String with 1st parameter removed.  2nd is bool for case sensitivity.
 #ifdef FORMAT_MONEY
    void Format(double dblNumber, size_t iFldLen, size_t iDecPlaces);                   // this contains dblNum converted to String, in iFldLen field width, with iDecPlaces to right of dec.
    void Format(double dblNum, int iFldWth, int iDecimal, bool blnRightJustified);      // same as above but allows right/left field justification
    void Format(size_t iNumber, int iFldLen, char cSeperator, bool blnRightJustified);  // For integral numbers; can specify justification, field width, and seperator for thousands place
    void Format(ssize_t iNumber, int iFldLen, char cSeperator, bool blnRightJustified); // Same as above for signed numbers
    void Money(double dblNum, size_t iFldLen, size_t iOffsetDollarSign);                // Specifically for money, can specify where the dollar sign goes in field
    void Money(double dblNum, size_t iFldLen, bool blnDollarSign);                      // Specifically for money, can specify whether a dollar sign is put before each amount
 #endif
 #ifdef CONSOLE_OUTPUT
    void Print(bool blnCrLf);
    void Print(const char* pStr, bool blnCrLf);
 #endif
 char* lpStr();
 int Len();                                                                             // accesor for String::iLen member
 int Capacity();                                                                        // Will be one less than underlying memory allocation
 ~String();   //String Destructor

 private:
 char*  lpBuffer;
 size_t iLen;
 size_t iCapacity;
};

String operator+(char* lhs, String& rhs);
#endif
